// to8bpp.c
// converts Plan 9 native image format to 8 bits per pixel, for use with Nuages
//	in progress



#include <u.h>
#include <libc.h>
#include <draw.h>
#include <memdraw.h>

#define CYAN 4
#define ALPHA 3
#define RED 2
#define GREEN 1
#define BLUE 0

static int  xsize, ysize;

void
usage(void)
{
	fprint(2, "usage: r2c  anaglyph_image \n");
	exits("usage");
}

void
version(void)
{
	fprint(2, "  r2c\n");
	fprint(2, "  swaps red and cyan channels of stereo images\n");
	fprint(2, "  Copy(L)eft 2003 Peter A. Cejchan\n");
	fprint(2, "  version 0.00\n");
	exits("version");
}

Memimage *
readimg(char *file)
{
	int fd;
	Rectangle rparam;
	Memimage *m;
	ulong tchan;
	char tmp[100];

	fd= 0;
	fd = open(file, OREAD);
	if(fd < 0)
		sysfatal("can't open %s: %r", file);


	memset(&rparam, 0, sizeof rparam);
	xsize  = ysize  = 0;

	m = readmemimage(fd);
	if(m == nil)
		sysfatal("can't read %s: %r", file);

	xsize = Dx(m->r);
	ysize = Dy(m->r);

	switch(m->chan){

	case RGB24:
	case RGBA32:
	case ARGB32:
	case XRGB32:
		return m;
		break;

	default:
		sysfatal("can't handle channel type %s", chantostr(tmp, m->chan));
	}
}

void
swapchans(uchar **in, int off, int iny, uchar **out, int outy, int n)
{
	int y, i, k, z;	
	z=off-n*(off/n);
	switch(n){
	case 3:
	case 4:
		for(y=0; y<outy; y++){
			if(z==RED){
if(in[y][off] < 128)
fputc(0, stdout) else fputc(0, stdout)

			}
		}
		break;
	default:
		sysfatal("can't handle  %d-channel images", n);	
	}
}

int
max(int a, int b)
{
	if(a > b)
		return a;
	return b;
}

Memimage*
red_to_cyan(char *file)
{
	int i, j, bpl, nchan;
	Memimage *new=nil, *m=nil;
	uchar **oscan, **nscan;



	/* read first original image from file */
	m = readimg(file);

	/* allocate resulting (new) image and scanlines*/
	new = allocmemimage(Rect(0, 0, xsize, ysize), m->chan);
	if(new == nil)
		sysfatal("can't allocate new image: %r");

	oscan = malloc(Dy(m->r)*sizeof(uchar*));
	nscan = malloc(max(ysize, Dy(m->r))*sizeof(uchar*));
	if(oscan == nil || nscan == nil)
		sysfatal("can't allocate: %r");

	/* unload first original image into scan lines */
	bpl = bytesperline(m->r, m->depth);
	for(i=0; i<Dy(m->r); i++){
		oscan[i] = malloc(bpl);
		if(oscan[i] == nil)
			sysfatal("can't allocate: %r");
		j = unloadmemimage(m, Rect(m->r.min.x, m->r.min.y+i, m->r.max.x, m->r.min.y+i+1), oscan[i], bpl);
		if(j != bpl)
			sysfatal("unloadmemimage");
	}

	/* allocate scan lines for destination. we do y first, so need at least Dy(m->r) lines */
	bpl = bytesperline(Rect(0, 0, xsize, Dy(m->r)), m->depth);
	for(i=0; i<max(ysize, Dy(m->r)); i++){
		nscan[i] = malloc(bpl);
		if(nscan[i] == nil)
			sysfatal("can't allocate: %r");
	}

	nchan = m->depth/8;

	/* extract  channels in Y direction and swap them*/
	for(i=0; i<xsize; i++)
		for(j=0; j<nchan; j++)
			swapchans(oscan, nchan*i+j, Dy(m->r), nscan, ysize, nchan);


	/* pack data into destination */
	bpl = bytesperline(new->r, m->depth);
	for(i=0; i<ysize; i++){
		j = loadmemimage(new, Rect(0, i, xsize, i+1), nscan[i], bpl);
		if(j != bpl)
			sysfatal("loadmemimage: %r");
	}
	return new;
}

void
main(int argc, char *argv[])
{
	int fd1, fd2, color;
	Memimage *new;
	char *file ;

		color=CYAN;
	ARGBEGIN{
	case 'c':
		color=CYAN;
		break;
	case 'g':
		color=GREEN;
		break;
	case 'b':
		color=BLUE;
		break;
	case 'v':
		version();
		break;
	default:
		exits("usage");
	}ARGEND

	if(argc < 1)
		usage();
	else if(argc == 1){
		file = argv[0];
	}
	memimageinit();

	new = nil;
	new = red_to_cyan(file);
	assert(new);

	if(writememimage(1, new) < 0)
		print("write error on output: %r");
	exits(nil);
}//main




